#!/bin/csh -f
#PBS -e en_cons.err
#PBS -o en_cons.log
#PBS -l nodes=1:ppn=3
#PBS -l walltime=20:00:00
#PBS -m ae
# Job name:
#PBS -N en_cons_cutoff
#PBS -r n


# if running as PBS job, change into the correct directory (the directory containing this file)
if ($?PBS_O_WORKDIR) then
cd $PBS_O_WORKDIR
endif

set pb = 16
set tp = 8
set num_blocks = 100
@ block_factor = 100


printf "Physics test: energy conservation/cut-off.\n"

set dt_list = (0.0016 0.0064)
set w_interval_list = (1600 400)
set cutoff_list = (2.5 4.0)

foreach i (1 2)
foreach j (1 2)
set dt = $dt_list[$i]
set cut = $cutoff_list[$j]
@ w_interval = $w_interval_list[$i]
@ block_size =  $w_interval * $block_factor


(../../../../UserPrograms/md_pair  --numIOBlocks=${num_blocks} --IOBlockSize=${block_size} --pb=$pb --tp=$tp --dt=${dt} --thermostat=0 --cutoff=${cut} --device=0 --dump_nrg=${w_interval} --writeTraj=0 --write_v_com_x=1 --mom_reset=0 > en_cons${cut}_${dt}.log) >& en_cons${cut}_${dt}.err

zcat TrajectoryFiles/energies*.dat.gz > energies${cut}_${dt}.dat
rm -rf TrajectoryFiles

end
end

grep "Processing time GPU" en_cons*.log > cpu_times.log
echo "To plot the energies, do: xmgrace -block energies2.5_0.0016.dat -bxy 0:6 -block energies2.5_0.0064.dat -bxy 0:6 -block energies4.0_0.0016.dat -bxy 0:6 -block energies4.0_0.0064.dat -bxy 0:6"

echo "To calculate energy fluctuations and drift, do: python ../Analysis.py [results will be in stdE_vs_dt.dat]"


